verbose = 'Show additional information.'
project = """
    The name of project.
"""
repository = """
    Repository which is used to store your files and directories.
"""
branch = """
    Branch of the repository used to push and pull from.
"""
check = """
    Whether check if repository exist.
"""
